#-----------------------------------------------------------------------
# Distribuição do p-valor em um teste cuja estatística de teste tem
# distribuição exata.

simul <- function() {
    x <- rnorm(20)
    y <- rnorm(20)
    t.test(x, y, var.equal = TRUE)$p.value
    # wilcox.test(x, y)$p.value
}
simul()

# P-valores do teste sob H0.
p <- replicate(1000, simul())

# Distribuição do teste é uniforme quando é exato.
plot(ecdf(p))
abline(a = 0, b = 1, col = 2)

#-----------------------------------------------------------------------
# O teste F tem distribuição exata?

g <- gl(5, 4)

simul <- function() {
    y <- rnorm(g)
    # plot(y ~ as.integer(g))
    m0 <- lm(y ~ g)
    anova(m0)[1, 5]
}

# P-valores do teste sob H0.
p <- replicate(1000, simul())

# Distribuição do teste é uniforme quando é exato.
plot(ecdf(p))
abline(a = 0, b = 1, col = 2)

#-----------------------------------------------------------------------
# O teste da deviance tem distribuição exata chi-quadrado?

g <- gl(2, 5)

simul <- function() {
    y <- rpois(g, lambda = 5)
    m0 <- glm(y ~ g, family = poisson)
    anova(m0, test = "Chi")[2, 5]
}

# P-valores do teste sob H0.
p <- replicate(1000, simul())

# Distribuição do teste é uniforme quando é exato.
plot(ecdf(p))
abline(a = 0, b = 1, col = 2)

#-----------------------------------------------------------------------
# Cobertura de intervalos de confiança.

# Gerando dados de um modelo de regressão linear simples.
x <- 0:10
X <- cbind(1, x)
beta <- c(3, 0.5)
y <- X %*% beta + rnorm(x, 0, 0.3)

plot(y ~ x)

m0 <- lm(y ~ x)
confint(m0)

simul <- function() {
    y <- X %*% beta + rnorm(x, 0, 0.3)
    m0 <- lm(y ~ x)
    confint(m0)
}

ic <- replicate(100, simul())
str(ic)

# Com i = 1 é o intercepto e i = 2 é a taxa.
i <- 1
plot(NULL, NULL, xlim = c(0, dim(ic)[3]), ylim = range(ic[i, , ]))
segments(1:ncol(ic[i, , ]),
         ic[i, 1, ],
         1:ncol(ic[i, , ]),
         ic[i, 2, ])
abline(h = beta[i])

# Aumentando o número de simulações
ic <- replicate(1000, simul())
str(ic)

# Conta quantos IC contem o valor paramétrico.
sum(ic[1, 1, ] < 3 & ic[1, 2, ] > 3)
sum(ic[2, 1, ] < 0.5 & ic[2, 2, ] > 0.5)

#-----------------------------------------------------------------------
# Taxa de cobertura em modelo não linear.

A <- 10
V <- 2
x <- 0:10
y <- A * x/(V + x) + rnorm(x, 0, 0.3)

plot(y ~ x)
curve(A * x/(V + x), add = TRUE)

n0 <- nls(y ~ A * x/(V + x), start = list(A = 10, V = 2))

# Intervalo de Wald.
confint.default(n0)

# Limite superior. Note que IC de Wald é est + z * stderror.
summary(n0)$coefficients[, 1] +
              qnorm(0.975) * summary(n0)$coefficients[, 2]

# Intervalo de perfil de verossimilhança.
confint(n0)

par(mfrow = c(1, 2))
plot(profile(n0))
layout(1)

# x <- 0:10
# X <- cbind(1, x)
# beta <- c(3, 0.5)
# y <- X %*% beta + rnorm(x, 0, 0.3)
#
# n0 <- nls(y ~ b0 + b1 * x, start = list(b0 = 3, b1 = 0.5))
# confint.default(n0)
# confint(n0)
#
# summary(n0)$coefficients[, 1] +
#               qnorm(0.975) * summary(n0)$coefficients[, 2]
#
# par(mfrow = c(1, 2))
# plot(profile(n0))
# layout(1)

simul <- function() {
    y <- A * x/(V + x) + rnorm(x, 0, 0.3)
    n0 <- nls(y ~ A * x/(V + x),
              start = list(A = 10, V = 2))
    confint.default(n0)
    # confint(n0)
}

ic <- replicate(100, simul())
str(ic)

# Com i = 1 é o intercepto e i = 2 é a taxa.
i <- 1
plot(NULL, NULL, xlim = c(0, dim(ic)[3]), ylim = range(ic[i, , ]))
segments(1:ncol(ic[i, , ]),
         ic[i, 1, ],
         1:ncol(ic[i, , ]),
         ic[i, 2, ])
abline(h = c(A, V)[i])

# Aumentando para ter uma estimativa mais precisa.
n <- 1000
ic <- replicate(n, simul())
str(ic)

# Taxas de cobertura.
sum(ic[1, 1, ] < A & ic[1, 2, ] > A)/n
sum(ic[2, 1, ] < V & ic[2, 2, ] > V)/n

# TODO: substitua o IC de Wald pelo IC de perfil de verossimilhança e
# repita tudo. Tire suas próprias conclusões.

#-----------------------------------------------------------------------
